﻿using System;
using System.Linq;
using System.Collections.Generic;
using CommonBehaviors.Actions;
using Styx;
using Styx.TreeSharp;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using Action = Styx.TreeSharp.Action;

namespace KBA.Helpers
{
    internal static class Item
    {
        private static LocalPlayer Me { get { return StyxWoW.Me; } }

        #region Items - General

        /// <summary>Checks the item cooldown is 0 and its usable</summary>
        private static bool CanUseItem(WoWItem item)
        {
            return item != null && item.Usable && item.Cooldown <= 0;
        }


        /// <summary>Uses the item specified (simulated click) </summary>
        private static void UseItem(WoWItem item)
        {
            item.Use();
        }

        #endregion Items - General

        #region Equiped Items

        /// <summary>Checks the equiped item cooldown is 0 and its usable</summary>
        private static bool CanUseEquippedItem(WoWItem item)
        {
            // Check for engineering tinkers!
            var itemSpell = Styx.WoWInternals.Lua.GetReturnVal<string>("return GetItemSpell(" + item.Entry + ")", 0);
            if (string.IsNullOrEmpty(itemSpell))
                return false;

            return item.Usable && item.Cooldown <= 0;
        }

        /// <summary>Creates a behavior to use an equipped item.</summary>
        private static Composite UseEquippedItem(uint slot)
        {
            return new PrioritySelector(
                ctx => StyxWoW.Me.Inventory.GetItemBySlot(slot),
                new Decorator(
                    ctx => ctx != null && CanUseEquippedItem((WoWItem)ctx),
                    new Action(ctx =>
                    {
                        UseItem((WoWItem)ctx);
                        return RunStatus.Failure;
                    })
                    ));
        }

        public static WoWItem FindFirstUsableItemById(uint ItemID)
        {
            List<WoWItem> carried = StyxWoW.Me.CarriedItems;
            return (from i in carried
                    where i != null &&
                          i.ItemInfo != null &&
                          i.Usable &&
                          i.Cooldown == 0 &&
                          i.ItemInfo.RequiredLevel <= StyxWoW.Me.Level &&
                          i.Entry == ItemID
                    orderby i.ItemInfo.Level descending
                    select i).FirstOrDefault();
        }
        public static Composite UseContainerItem(uint id)
        {
            return new PrioritySelector(
                ctx => FindFirstUsableItemById(id),
                new Decorator(
                    ret => ret != null,
                    new Sequence(
                        new Action(ret => Logger.InfoLog(String.Format("Using {0} @ {1:F1}% Health", ((WoWItem)ret).Name, StyxWoW.Me.HealthPercent))),
                        new Action(ret => { ((WoWItem)ret).UseContainerItem(); return RunStatus.Failure; }),
                        Spell.CreateWaitForLagDuration()))
                        );
        }
        /// <summary>Creates a behavior to use an item, in your bags or paperdoll.</summary>
        public static Composite UseItem(uint id)
        {
            return new PrioritySelector(
                ctx => ObjectManager.GetObjectsOfType<WoWItem>().FirstOrDefault(item => item.Entry == id),
                new Decorator(
                    ctx => ctx != null && CanUseItem((WoWItem)ctx),
                    new Action(ctx =>
                    {
                        UseItem((WoWItem)ctx);
                        return RunStatus.Failure;
                    })));
        }

        public static WoWItem FindFirstUsableItemBySpell(params string[] spellNames)
        {
            List<WoWItem> carried = StyxWoW.Me.CarriedItems;
            var spellNameHashes = new HashSet<string>(spellNames);

            return (from i in carried
                    let spells = i.ItemSpells
                    where i.ItemInfo != null && spells != null && spells.Count != 0 &&
                          i.Usable &&
                        // ReSharper disable CompareOfFloatsByEqualityOperator
                          i.Cooldown == 0 &&
                        // ReSharper restore CompareOfFloatsByEqualityOperator
                          i.ItemInfo.RequiredLevel <= StyxWoW.Me.Level &&
                          spells.Any(s => s.IsValid && s.ActualSpell != null && spellNameHashes.Contains(s.ActualSpell.Name))
                    orderby i.ItemInfo.Level descending
                    select i).FirstOrDefault();
        }

        /// <summary>Uses the players engineering gloves provided the player has engineering gloves
        /// and the engineering gloves are not on cooldown and the engineering gloves are usable.</summary>
        public static Composite UseEngineerGloves()
        {
            return new PrioritySelector(
                ctx => StyxWoW.Me.Inventory.GetItemBySlot((uint)WoWInventorySlot.Hands),
                new Decorator(
                    ctx => ctx != null && CanUseEquippedItem((WoWItem)ctx),
                    new Action(ctx =>
                    {
                        UseItem((WoWItem)ctx);
                        return RunStatus.Failure;
                    })
                    )
               );
        }

        #endregion Equiped Items

        #region Bag Items

        /// <summary>Returns true if the player has a mana gem in his bag.</summary>
        public static bool HaveManaGem()
        {
            return ObjectManager.GetObjectsOfType<WoWItem>(false).Any(item => item.Entry == 36799 || item.Entry == 81901);
        }

        /// <summary>Returns true if the player has the item in his bag.</summary>
        public static bool HasItemInBag(WoWItem item)
        {
            return item != null && Me.BagItems.Any(x => x.ItemInfo.Id == item.ItemInfo.Id);
        }

        /// <summary>Returns true if the player has the item in his bag.</summary>
        public static bool HasCarriedItem(WoWItem item)
        {
            return item != null && Me.CarriedItems.Any(x => x.ItemInfo.Id == item.ItemInfo.Id);
        }

        /// <summary>Attempts to use the bag item provided it is usable and its not on cooldown</summary>
        /// <param name="name">the name of the bag item to use</param>
        /// <param name="cond">The conditions that must be true</param>
        /// <param name="label">A descriptive label for the clients GUI logging output</param>
        /// <returns>The use bag item.</returns>
        public static Composite UseBagItem(string name, CanRunDecoratorDelegate cond, string label)
        {
            WoWItem item = null;
            return new Decorator(
                delegate(object a)
                {
                    Logger.InfoLog("try to use item with name {0}", name);
                    if (!cond(a))
                        return false;
                    item = Me.BagItems.FirstOrDefault(x => x.Name == name && x.Usable && x.Cooldown <= 0);
                    return item != null;
                },
                new Sequence(
                    new Action(a => Logger.InfoLog(" [BagItem] {0} ", label)),
                    new Action(a => item.UseContainerItem())));
        }

        /// <summary>Attempts to use the bag item provided it is usable and its not on cooldown</summary>
        /// <param name="name">the name of the bag item to use</param>
        /// <param name="cond">The conditions that must be true</param>
        /// <param name="label">A descriptive label for the clients GUI logging output</param>
        /// <returns>The use bag item.</returns>
        public static Composite UseBagItem(int id, CanRunDecoratorDelegate cond, string label)
        {
            WoWItem item = null;
            return new Decorator(
                delegate(object a)
                {
                    Logger.InfoLog("try to use item with id {0}", id);
                    if (!cond(a))
                        return false;
                    item = Me.BagItems.FirstOrDefault(x => x.Entry == id && x.Usable && x.Cooldown <= 0);
                    return item != null;
                },
                new Sequence(
                    new Action(a => Logger.InfoLog(" [BagItem] {0} ", label)),
                    new Action(a => item.UseContainerItem())));
        }

        #endregion Bag Items

        #region Trinkets


        #region PotionTestShit

        private static Composite CreateWaitForLagDuration()
        {
            return new WaitContinue(TimeSpan.FromMilliseconds((StyxWoW.WoWClient.Latency * 2) + 150), ret => false, new ActionAlwaysSucceed());
        }

        #endregion PotionTestShit


        #endregion Trinkets

        #region Potions

        public static uint EnsureAppropriatePotion(bool cond)
        {
            if (!cond) return 0;
            uint PotionId = 0;
            SpecType s = Me.SpecType;
            WoWSpec t = Me.Specialization;
            WoWClass c = Me.Class;
            double m = Me.ManaPercent;
            if (s == SpecType.Tank)
            {
                if (c == WoWClass.Monk)
                {
                    PotionId = 76089;
                }
                else
                {
                    PotionId = 76090;
                }
            }
            if (s == SpecType.MeleeDps)
            {
                if (c == WoWClass.DeathKnight || c == WoWClass.Paladin || c == WoWClass.Warrior)
                {
                    PotionId = 76090;
                }
                else
                {
                    PotionId = 76089;
                }
            }

            if (s == SpecType.RangedDps)
            {
                if (c == WoWClass.Hunter)
                {
                    PotionId = 76089;
                }
                else
                {
                    PotionId = 76093;
                }
            }
            if (s == SpecType.Healer)
            {
                if (m < 30)
                {
                    PotionId = 76098;
                }
                else
                {
                    PotionId = 76093;
                }
            }
            return PotionId;
        }

        #endregion Potions

        #region Tier Piece Armor Detection

        /// <summary>Checks the number of Tier pieces a player is wearing </summary>
        private static int NumTierPieces(int txxItemSetId)
        {
            {
                var
                count = Me.Inventory.Equipped.Hands != null && Me.Inventory.Equipped.Hands.ItemInfo.ItemSetId != 0 && Me.Inventory.Equipped.Hands.ItemInfo.ItemSetId == txxItemSetId ? 1 : 0;
                count += Me.Inventory.Equipped.Legs != null && Me.Inventory.Equipped.Legs.ItemInfo.ItemSetId != 0 && Me.Inventory.Equipped.Legs.ItemInfo.ItemSetId == txxItemSetId ? 1 : 0;
                count += Me.Inventory.Equipped.Chest != null && Me.Inventory.Equipped.Chest.ItemInfo.ItemSetId != 0 && Me.Inventory.Equipped.Chest.ItemInfo.ItemSetId == txxItemSetId ? 1 : 0;
                count += Me.Inventory.Equipped.Shoulder != null && Me.Inventory.Equipped.Shoulder.ItemInfo.ItemSetId != 0 && Me.Inventory.Equipped.Shoulder.ItemInfo.ItemSetId == txxItemSetId ? 1 : 0;
                count += Me.Inventory.Equipped.Head != null && Me.Inventory.Equipped.Head.ItemInfo.ItemSetId != 0 && Me.Inventory.Equipped.Head.ItemInfo.ItemSetId == txxItemSetId ? 1 : 0;
                return count;
            }
        }

        /// <summary>Returns true if player has 2pc Tier set Bonus</summary>
        public static bool Has2PcTeirBonus(int txxItemSetId)
        {
            {
                return NumTierPieces(txxItemSetId) >= 2;
            }
        }

        /// <summary>Returns true if player has 4pc Tier set Bonus</summary>
        public static bool Has4PcTeirBonus(int txxItemSetId)
        {
            {
                return NumTierPieces(txxItemSetId) == 4;
            }
        }

        #endregion Tier Piece Armor Detection
    }
}
